﻿using System;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Web.Security;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using NHProviderTestHelpers;

namespace NHibernateProvider.Tests.Unit
{
    [TestClass]
    public class NHibernateMembershipProviderTestsEncrypted
    {
        private NHibernateMembershipProvider _nhProv;

        #region Setup and Teardown
        [TestInitialize()]
        public void InitializeTest()
        {
            _nhProv = new NHibernateMembershipProvider();
            _nhProv.UserRepo = Utilities.GetMockRepository();
            var config = new NameValueCollection();
            config.Add("requiresQuestionAndAnswer", "true");
            config.Add("passwordFormat", "Encrypted");
            _nhProv.Initialize("", config);
        }

        [TestCleanup()]
        public void CleanupTest()
        {
            _nhProv = null;
        }
        #endregion

        [TestMethod]
        [ExpectedException(typeof(MembershipPasswordException))]
        public void GetPassword_GivenGoodUserAndBadAnswer_WithRequireAnswer_ThrowsException()
        {
            //Arrange
            var name = "EncryptUser";
            var answer = "BadAnswer";
            //Act
            _nhProv.GetPassword(name, answer);
            //Assert
        }

        [TestMethod]
        public void ChangePassword_GoodUserGoodPass_ReturnsTrue()
        {
            //Arrange
            var user = "EncryptUser";
            var oldpass = "GoodPass";
            var newpass = "ABC123!?";
            //Act
            var actual = _nhProv.ChangePassword(user, oldpass, newpass);
            //Assert
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void GetPassword_GivenGoodUserAndGoodAnswer_WithRequireAnswer_ReturnsPassword()
        {
            //Arrange
            var name = "EncryptUser";
            var answer = "GoodAnswer";
            var expected = "GoodPass";
            //Act
            var actual = _nhProv.GetPassword(name, answer);
            //Assert
            Assert.AreEqual(expected,actual);
        }

        [TestMethod]
        public void GetPassword_GivenGoodUserAndGoodAnswer_ReturnsPassword()
        {
            //Arrange
            var name = "EncryptUser";
            var answer = "GoodAnswer";
            var expected = "GoodPass";
            //Act
            var actual = _nhProv.GetPassword(name, answer);
            //Assert
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        [ExpectedException(typeof(ProviderException))]
        public void ResetPassword_NullAnswer_QandARequired_ThrowsException()
        {
            //Arrange
            var name = "GoodUser";
            //Act
            _nhProv.ResetPassword(name, null);
            //Assert
        }

        [TestMethod]
        [ExpectedException(typeof(MembershipPasswordException))]
        public void ResetPassword_BadAnswer_QandARequired_ThrowsException()
        {
            //Arrange
            var name = "EncryptUser";
            var answer = "BadAnswer";
            //Act
            _nhProv.ResetPassword(name, answer);
            //Assert
        }

        [TestMethod]
        public void ResetPassword_GoodUser_QandARequired_ReturnsNewPassword()
        {
            //Arrange
            var name = "EncryptUser";
            var answer = "GoodAnswer";
            //Act
            var actual = _nhProv.ResetPassword(name, answer);
            //Assert
            Assert.AreNotEqual("", actual);
        }
    }
}
