﻿using System;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using NHProviderTestHelpers;

namespace NHibernateProvider.Tests.Unit
{
    [TestClass]
    public class NHibernateMembershipProviderTestsHashed
    {
        private NHibernateMembershipProvider _nhProv;

        #region Setup and Teardown
        [TestInitialize()]
        public void InitializeTest()
        {
            _nhProv = new NHibernateMembershipProvider();
            _nhProv.UserRepo = Utilities.GetMockRepository();
            var config = new NameValueCollection();
            config.Add("passwordFormat", "Hashed");
            config.Add("requiresQuestionAndAnswer", "true");
            _nhProv.Initialize("", config);
        }

        [TestCleanup()]
        public void CleanupTest()
        {
            _nhProv = null;
        }
        #endregion

        [TestMethod]
        [ExpectedException(typeof(ProviderException))]
        public void GetPassword_GivenGoodUserAndGoodAnswer_ThrowsException()
        {
            //Arrange
            var name = "GoodUser";
            var answer = "GoodAnswer";
            //Act
            _nhProv.GetPassword(name, answer);
        }

        [TestMethod]
        public void ResetPassword_GoodUser_QandARequired_ReturnsNewPassword()
        {
            //Arrange
            var name = "HashUser";
            var answer = "GoodAnswer";
            //Act
            var actual = _nhProv.ResetPassword(name, answer);
            //Assert
            Assert.AreNotEqual("", actual);
        }

        [TestMethod]
        public void ValidateUser_GivenGoodUserGoodPassword_ReturnsTrue()
        {
            //Arrange
            var userName = "HashUser";
            var userPass = "GoodPass";
            //Act
            var actual = _nhProv.ValidateUser(userName, userPass);
            //Assert
            Assert.IsTrue(actual);
        }
    }
}
